plugins {
	id("fabric-loom") version "0.2.7-SNAPSHOT"
}

java {
	sourceCompatibility = JavaVersion.VERSION_1_8
	targetCompatibility = JavaVersion.VERSION_1_8
}

base.archivesBaseName = project.property("archives_base_name") as String
val modVersion = project.property("mod_version")
version = "${modVersion}+${project.property("minecraft_version")}"
group = project.property("maven_group")!!

minecraft {
}

repositories {
	maven {
		url = uri("https://maven.thebrokenrail.com/")
	}
}

dependencies {
	minecraft("com.mojang:minecraft:${project.property("minecraft_version")}")
	mappings("net.fabricmc:yarn:${project.property("minecraft_version")}+build.${project.property("yarn_build")}:v2")
	modImplementation("net.fabricmc:fabric-loader:${project.property("fabric_loader_version")}")

	modImplementation("com.thebrokenrail:scriptcraft:${project.property("minecraft_version")}")
	include("com.thebrokenrail:scriptcraft:${project.property("minecraft_version")}")
}

tasks.named<Copy>("processResources") {
	inputs.property("version", modVersion)

	from(sourceSets["main"].resources.srcDirs) {
		include("fabric.mod.json")
		expand("version" to modVersion)
	}

	from(sourceSets["main"].resources.srcDirs) {
		exclude("fabric.mod.json")
	}
}

// ensure that the encoding is set to UTF-8, no matter what the system default is
// this fixes some edge cases with special characters not displaying correctly
// see http://yodaconditions.net/blog/fix-for-java-file-encoding-problems-with-gradle.html
tasks.withType<JavaCompile> {
	options.encoding = "UTF-8"
}

// Loom will automatically attach sourcesJar to a RemapSourcesJar task and to the "build" task
// if it is present.
// If you remove this task, sources will not be generated.
tasks.register<Jar>("sourcesJar") {
	archiveClassifier.set("sources")
	from(sourceSets["main"].allSource)
}
artifacts {
	add("archives", tasks["sourcesJar"])
}

tasks.named<Jar>("jar") {
	from("LICENSE")
}
